import logging


from django.shortcuts import render


from pwdtk.helpers import PwdtkSettings

logger = logging.getLogger(__name__)


def lockout_response(request, pwdtk_data):
    """ create login response for locked out users
    """

    age = pwdtk_data.fail_age
    to_wait = PwdtkSettings.PWDTK_LOCKOUT_TIME - age
    to_wait_minutes, to_wait_seconds = divmod(to_wait, 60)
    to_wait_str = "%i minutes and %i seconds" % (
        to_wait_minutes, to_wait_seconds)
    lockout_context = {
        "to_wait": to_wait_str,
        'failure_limit': pwdtk_data.failed_logins,
        'to_wait_time_tuple': (to_wait_minutes, to_wait_seconds),
    }

    return render(
        request,
        PwdtkSettings.PWDTK_LOCKOUT_TEMPLATE,
        lockout_context,
        status=403)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


def uniqify_pwd_data(apps, schema_editor):
    """ finds multiple entries for same username and keeps only one

        This is required prior to adding a unique contraint to
        the PwdData / unique field
    """
    PwdData = apps.get_model("pwdtk", "PwdData")
    usr_w_cnt = PwdData.objects.all().values('username').annotate(
        ucnt=models.Count("username")).filter(ucnt__gt=1)
    for username in usr_w_cnt.values_list("username", flat=True):
        print("removing redundant PwdData entries for %r" % username)
        to_rm = PwdData.objects.filter(username=username).order_by("pk")[1:]
        for usr in to_rm:
            usr.delete()


class Migration(migrations.Migration):

    dependencies = [
        ('pwdtk', '0001_initial'),
    ]

    operations = [
        migrations.RunPython(uniqify_pwd_data),
    ]



import logging

from django.apps import AppConfig
from django.contrib.auth import get_user_model

from pwdtk.helpers import PwdtkSettings

logger = logging.getLogger(__name__)


logger.debug("imp pwdtk APPS")  # added trace for dbg of dj 1.8 -> dj 1.11


class PwdTkConfig(AppConfig):
    name = 'pwdtk'

    def ready(self):
        """ Install all required hooks for pwdtk
            The required hooks depend on the django version.

        """
        logger.debug("PWDTK READY")
        if not PwdtkSettings.PWDTK_ENABLED:
            logger.debug("PWDTK DISABLED")
            return

        User = get_user_model()
        from pwdtk.watchers import watch_set_password
        User.set_password = watch_set_password(User.set_password)



from __future__ import absolute_import
from __future__ import print_function

import importlib
import inspect
import six

from django.conf import settings


def recursion_depth():
    """return recursion depth. 0 if no recursion"""
    # taken from https://github.com/looking-for-a-job/recursion-detect.py
    counter = 0
    frames = inspect.getouterframes(inspect.currentframe())[1:]
    top_frame = inspect.getframeinfo(frames[0][0])
    for frame, _, _, _, _, _ in frames[1:]:
        (path, line_number, func_name, lines, index) = inspect.getframeinfo(
            frame)
        if path == top_frame[0] and func_name == top_frame[2]:
            counter += 1
    return counter


def _resolve_object_path(dotted_name):

    if isinstance(dotted_name, six.string_types):

        module_name, obj_name = dotted_name.rsplit(".", 1)
        module = importlib.import_module(module_name)
        return getattr(module, obj_name)

    return dotted_name


class PwdtkSettingsType(type):

    def __getattr__(cls, key):
        return getattr(settings, key)


if getattr(settings, 'PWDTK_CUSTOM_SETTINGS_CLS', None):
    class PwdtkSettings(six.with_metaclass(
      _resolve_object_path(settings.PWDTK_CUSTOM_SETTINGS_CLS))):
        pass
else:
    class PwdtkSettings(six.with_metaclass(PwdtkSettingsType)):
        pass



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('pwdtk', '0006_auto_20200304_1117'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='pwddata',
            name='data',
        ),
        migrations.RemoveField(
            model_name='pwddata',
            name='old_user_id',
        ),
        migrations.RemoveField(
            model_name='pwddata',
            name='username',
        ),
        migrations.AlterField(
            model_name='pwddata',
            name='user',
            field=models.OneToOneField(
                related_name='pwdtk_data',
                default=None,
                to=settings.AUTH_USER_MODEL),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='pwddata',
            name='must_renew',
            field=models.BooleanField(default=False),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import datetime
from django.conf import settings
import jsonfield.fields


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('pwdtk', '0004_auto_20200304_1114'),
    ]

    operations = [
        migrations.AddField(
            model_name='pwddata',
            name='fail_time',
            field=models.DateTimeField(null=True),
        ),
        migrations.AddField(
            model_name='pwddata',
            name='failed_logins',
            field=models.PositiveIntegerField(default=0),
        ),
        migrations.AddField(
            model_name='pwddata',
            name='last_change_time',
            field=models.DateTimeField(default=datetime.datetime.utcnow),
        ),
        migrations.AddField(
            model_name='pwddata',
            name='locked',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='pwddata',
            name='password_history',
            field=jsonfield.fields.JSONField(default=[]),
        ),
        migrations.AddField(
            model_name='pwddata',
            name='user',
            field=models.OneToOneField(
                related_name='pwdtk_data',
                null=True,
                to=settings.AUTH_USER_MODEL),
        ),
    ]



import logging

logger = logging.getLogger(__name__)


def add_backend(backends):
    """ adds auth backend depending on django version
    """

    backends.insert(0, 'pwdtk.auth_backends.PwdtkBackend')


def add_middlewares(
  middlewares,
  after='django.contrib.auth.middleware.AuthenticationMiddleware'):
    """ adds middlewares depending on django.VERSION """

    middlewares.append('pwdtk.middlewares.PwdtkMiddleware')


# PWDTK Common settings for login / logout views
# ----------------------------------------------------------

# Allow to completely enable / disable pwdtk
# This should disable all hooks / middlewares and auth backends
PWDTK_ENABLED = True

# Model to be used for storing PWDTK related info
PWDTK_USER_PARAMS = 'pwdtk.auth_backends_data.UserData'

# to know what login views to watch
# for django < 1.11
PWDTK_LOGIN_VIEW = 'django.contrib.auth.views.login'

# for django > 1.11
PWDTK_LOGIN_VIEW_CLASS = 'django.contrib.auth.views.LoginView'


# PWDTK Passsword lockout after logins with bad password
# ----------------------------------------------------------


# Time in seconds to lockout a user who entered a bad password too iften
PWDTK_LOCKOUT_TIME = 60

# Amount of bad passwords before a user is locked out
PWDTK_USER_FAILURE_LIMIT = 3

# the field in the login form, that contains the user name
PWDTK_USERNAME_FORM_FIELD = 'username'

# the field in the login form, that contains the password
# PWDTK_PASSWORD_FORM_FIELD = 'username'

PWDTK_IP_FAILURE_LIMIT = 0


# PWDTK force password renewal
# ------------------------------
# should ckeck code of https://github.com/tarak/django-password-policies to
# see, that we use a similiar config.

# age of passwords before it has to be renewed
PWDTK_PASSWD_AGE = 30 * 24 * 60 * 60


# amount of passwords before and old password can be reused
PWDTK_PASSWD_HISTORY_LEN = 3

# template to display for password renewal
PWDTK_PASSWD_CHANGE_VIEW = "password_change"


PWDTK_PASSWD_CHANGE_ALLOWED_PATHS = []

# Name of the template to be used when a user is locked out
PWDTK_LOCKOUT_VIEW = None
PWDTK_LOCKOUT_TEMPLATE = 'login/locked_out_simple.html'

# for PWDTK unit testing.
# --------------------------

# Url of the admin page
PWDTK_TEST_ADMIN_URL = "/admin/"

# message to be found in contents to detect a bad login
PWDTK_TEST_LOGIN_FAIL_SUBMSG = b'Please enter the correct'

# message to be found in contents to detect a locked out user
PWDTK_TEST_LOCKOUT_SUBMSG = b'So bad!'



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('pwdtk', '0003_auto_20191107_1501'),
    ]

    operations = [
        migrations.RenameField(
            model_name='pwddata',
            old_name='user_id',
            new_name='old_user_id',
        ),
    ]



import json
import logging

from six.moves.urllib.parse import urlencode

import django
from django.core.exceptions import MiddlewareNotUsed
from django.core.urlresolvers import reverse
from django.http import HttpResponse
from django.shortcuts import redirect

try:
    from django.utils.deprecation import MiddlewareMixin
except: # noqa E722
    class MiddlewareMixin(object):
        pass


from pwdtk.auth_backends import PwdtkLockedException
from pwdtk.helpers import PwdtkSettings
from pwdtk.models import PwdData
from pwdtk.views import lockout_response


logger = logging.getLogger(__name__)


class PwdtkMiddleware(MiddlewareMixin):

    def __init__(self, get_response=None):

        if not PwdtkSettings.PWDTK_ENABLED:
            logger.debug("PWDTK middleware is disabled")
            raise MiddlewareNotUsed("pwdtk is disabled")
        if get_response:
            super(PwdtkMiddleware, self).__init__(
                get_response=get_response)
        else:
            super(PwdtkMiddleware, self).__init__()

    def must_renew_password(self, request):

        if request.method == 'GET':
            return False

        if django.VERSION < (1, 10):
            is_authenticated = request.user and request.user.is_authenticated()
        else:
            is_authenticated = request.user and request.user.is_authenticated

        if not is_authenticated:
            return False

        if not PwdData.get_or_create_for_user(request.user).must_renew:
            return False

        if request.path == reverse(PwdtkSettings.PWDTK_PASSWD_CHANGE_VIEW):
            return False

        if request.path in PwdtkSettings.PWDTK_PASSWD_CHANGE_ALLOWED_PATHS:
            return False

        return True

    def process_exception(self, request, exception):

        if isinstance(exception, PwdtkLockedException):
            context = exception.pwdtk_data.get_lockout_context()
            if request.is_ajax():
                return HttpResponse(
                    json.dumps(context),
                    content_type='application/json',
                    status=403,
                    )

            if PwdtkSettings.PWDTK_LOCKOUT_VIEW:
                return redirect("%s?%s" % (
                    reverse(PwdtkSettings.PWDTK_LOCKOUT_VIEW),
                    urlencode(context)
                    )
                )
            return lockout_response(request, exception.pwdtk_data)
        return None

    def process_request(self, request):

        if self.must_renew_password(request):
            return redirect(reverse(PwdtkSettings.PWDTK_PASSWD_CHANGE_VIEW))

    def process_response(self, request, response):

        if self.must_renew_password(request):
            return redirect(reverse(PwdtkSettings.PWDTK_PASSWD_CHANGE_VIEW))

        return response



import os
BASE_DIR = os.path.realpath(os.path.dirname(__file__))
TOP_DIR = os.path.dirname(os.path.dirname(BASE_DIR))

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'basic': {
            'format': '%(levelname)s %(name)s %(message)s',
            }
        },
    'handlers': {
        'console': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
            'formatter': 'basic',
        },
    },
    'loggers': {
        '': {
            'handlers': ['console'],
            'level': 'DEBUG',
            'propagate': True,
        },
        'django.db.backends': {
            'level': 'INFO',
            },
        'django.db.backends.schema': {
            'level': 'WARNING',
            },
    },
}

log_cfg = os.path.join(TOP_DIR, 'log_settings.py')
if os.path.isfile(log_cfg):
    from log_settings import LOGGING  # noqa: F401

del log_cfg
del BASE_DIR
del TOP_DIR



import logging
import os

from functools import wraps

from django.utils import timezone

from pwdtk.helpers import recursion_depth
from pwdtk.helpers import PwdtkSettings
from pwdtk.signals import pwd_data_post_change_password


logger = logging.getLogger(__name__)
LOG_PASSWORDS = os.environ.get(
    'PWDTK_LOG_PASSWORDS', 'no')[:1].lower() in 'yt1'


def watch_set_password(orig_set_password):
    """ decorator for set_password functions to
        detect password changes
    """

    if hasattr(orig_set_password, '_decorated_by_pwdtk'):
        logger.warning("set_password already decorated by pwdtk")
        return orig_set_password

    logger.debug("decorating %s", repr(orig_set_password))

    # from pwdtk.auth_backends import MHPwdPolicyBackend
    # UserData = MHPwdPolicyBackend.get_user_data_cls()

    @wraps(orig_set_password)
    def decorated(self, password):
        passwd_str = password if LOG_PASSWORDS else '********'
        logger.debug(
            "potential passwd change detected %r %s",
            self.username,
            repr(passwd_str),
            )
        # logging in if a user has a pwd hash, that isn't default
        # causes a weird recursion
        depth = recursion_depth()

        if not self.username:
            logger.debug("no username specified. Watcher has nothing to do")
            return orig_set_password(self, password)
        if depth > 0:
            # TODO: instead of detecting a recursion we might try to detect,
            # whether this function si being called during a login. This
            # might be less obscure but perhaps less safe. detecting
            # recursions will always avoid endless recursions
            logger.debug("recursion detected. abandon pwd_change intercept")
            return orig_set_password(self, password)
        changed = not self.check_password(password)
        logger.debug("changed = %s", changed)
        if changed:
            logger.debug("password change detected")
            orig_set_password(self, password)
            if hasattr(self, 'pwdtk_data'):
                now = timezone.now()
                self.pwdtk_data.password_history.insert(
                    0, (now, self.password))
                self.pwdtk_data.password_history[
                    PwdtkSettings.PWDTK_PASSWD_HISTORY_LEN:] = []
                self.pwdtk_data.last_change_time = now
                self.pwdtk_data.must_renew = False
                self.pwdtk_data.save()
                pwd_data_post_change_password.send(
                    sender=self.pwdtk_data.__class__,
                    pwd_data=self.pwdtk_data)
        return orig_set_password(self, password)

    decorated._decorated_by_pwdtk = True

    return decorated



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import jsonfield.fields


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='PwdData',
            fields=[
                ('id', models.AutoField(
                    verbose_name='ID', serialize=False,
                    auto_created=True, primary_key=True)),
                ('user_id', models.PositiveIntegerField(
                    default=0, null=True, blank=True)),
                ('username', models.CharField(
                    default=b'', max_length=80, null=True, blank=True)),
                ('data', jsonfield.fields.JSONField(default={})),
            ],
        ),
    ]



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function


import logging

from django.contrib.auth import get_user_model
from django.contrib.auth.backends import ModelBackend

from pwdtk.helpers import PwdtkSettings
from pwdtk.models import PwdData

logger = logging.getLogger(__name__)


class PwdtkLockedException(Exception):
    """ custom exception """

    def __init__(self, pwdtk_data):

        self.pwdtk_data = pwdtk_data


class PwdtkBackend(ModelBackend):
    """
    Authenticates against settings.AUTH_USER_MODEL.
    """
    def authenticate(self, username=None, password=None, **kwargs):
        UserModel = get_user_model()
        if username is None:
            username = kwargs.get(UserModel.USERNAME_FIELD)
        try:
            user = UserModel._default_manager.get_by_natural_key(username)
            pwdtk_data = PwdData.get_or_create_for_user(user)
            if pwdtk_data.is_locked():
                raise PwdtkLockedException(pwdtk_data)
            if user.check_password(password):
                must_renew = pwdtk_data.compute_must_renew()
                if (pwdtk_data.failed_logins or pwdtk_data.fail_time or
                   pwdtk_data.locked or must_renew != pwdtk_data.must_renew):
                    pwdtk_data.failed_logins = 0
                    pwdtk_data.fail_time = None
                    pwdtk_data.locked = False
                    pwdtk_data.must_renew = must_renew
                    pwdtk_data.save()
                return user
            else:
                if PwdtkSettings.PWDTK_USER_FAILURE_LIMIT is None:
                    return None
                if (pwdtk_data.failed_logins+1 >=
                   PwdtkSettings.PWDTK_USER_FAILURE_LIMIT):
                    pwdtk_data.set_locked(pwdtk_data.failed_logins+1)
                    raise PwdtkLockedException(pwdtk_data)
                else:
                    pwdtk_data.failed_logins += 1
                    pwdtk_data.save()
        except UserModel.DoesNotExist:
            # Run the default password hasher once to reduce the timing
            # difference between an existing and a non-existing user (#20760).
            UserModel().set_password(password)
            # TODO: With this policy we can guess if a username exists,
            # we must also block non-existent username accesses



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations


def fill_pwd_data(apps, schema_editor):

    PwdData = apps.get_model("pwdtk", "pwddata")
    for pwd_data in PwdData.objects.all():
        pwd_data.user_id = pwd_data.old_user_id
        pwd_data.locked = pwd_data.data.get("locked", False)
        pwd_data.failed_logins = pwd_data.data.get("failed_logins", 0)
        pwd_data.fail_time = pwd_data.data.get("fail_time")
        pwd_data.password_history = pwd_data.data.get('passwd_history', [])
        if pwd_data.password_history:
            pwd_data.last_change_time = pwd_data.password_history[0][0]
        pwd_data.save()


class Migration(migrations.Migration):

    dependencies = [
        ('pwdtk', '0005_auto_20200304_1116'),
    ]

    operations = [
        migrations.RunPython(
            fill_pwd_data,
            reverse_code=migrations.RunPython.noop),
    ]



class PytestTestRunner(object):
    """Runs pytest to discover and run tests."""

    def __init__(self, verbosity=1, failfast=False, keepdb=False, **kwargs):
        self.verbosity = verbosity
        self.failfast = failfast
        self.keepdb = keepdb

    def run_tests(self, test_labels):
        """Run pytest and return the exitcode.

        It translates some of Django's test command option to pytest's.
        """
        import pytest

        argv = []
        if self.verbosity == 0:
            argv.append('--quiet')
        if self.verbosity == 2:
            argv.append('--verbose')
        if self.verbosity == 3:
            argv.append('-vv')
        if self.failfast:
            argv.append('--exitfirst')
        if self.keepdb:
            argv.append('--reuse-db')

        argv.extend(test_labels)
        return pytest.main(argv)



from __future__ import absolute_import

import jsonfield
import pytz

from django.conf import settings
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils import timezone

from pwdtk.helpers import PwdtkSettings

AUTH_USER_MODEL = getattr(settings, 'AUTH_USER_MODEL', 'auth.User')


@python_2_unicode_compatible
class PwdData(models.Model):
    """ a model in case no custom way is specified for storing related data
    """
    user = models.OneToOneField(AUTH_USER_MODEL, related_name='pwdtk_data')
    locked = models.BooleanField(default=False)
    failed_logins = models.PositiveIntegerField(default=0)
    fail_time = models.DateTimeField(null=True)
    must_renew = models.BooleanField(default=False)
    last_change_time = models.DateTimeField(default=timezone.now)
    password_history = jsonfield.JSONField(default=[])

    @classmethod
    def get_or_create_for_user(cls, user):
        if not hasattr(user, 'pwdtk_data'):
            user.pwdtk_data = cls.objects.create(user=user)

        return user.pwdtk_data

    def __str__(self):
        return("%r, %r" % (self.user.id, self.user.username))

    def set_locked(self, failed_logins=None):
        if failed_logins is not None:
            self.failed_logins = failed_logins
        self.locked = True
        self.fail_time = timezone.now()
        self.save()

    def is_locked(self):
        """ determines whether a user is still locked out
        """
        if not self.locked:
            return False

        if self.fail_age < PwdtkSettings.PWDTK_LOCKOUT_TIME:
            return True

        self.locked = False
        self.save()
        return False

    @property
    def aware_fail_time(self):
        if timezone.is_aware(self.fail_time):
            return self.fail_time
        else:
            return pytz.timezone(
                PwdtkSettings.TIME_ZONE).localize(self.fail_time)

    @property
    def fail_age(self):

        return int((timezone.now() - self.fail_time).total_seconds())

    def compute_must_renew(self):
        """ determines whether a user must renew his password
        """
        if getattr(self.user, "disable_must_renew", False):
            return False
        return ((timezone.now() - self.last_change_time).total_seconds() >
                PwdtkSettings.PWDTK_PASSWD_AGE)

    def get_lockout_context(self):

        return {
            "username": self.user.username,
            "lockout_time": PwdtkSettings.PWDTK_LOCKOUT_TIME,
            "failed_logins": self.failed_logins,
            "fail_time": self.aware_fail_time.isoformat(),
        }



from __future__ import absolute_import
from __future__ import print_function

import importlib

import pytest

from django.conf import settings
from django.contrib.auth import get_user_model
from pwdtk.tests.fixtures import two_users  # noqa: F401


User = get_user_model()


@pytest.mark.django_db
def test_pwd_nodflt_hash(two_users):  # noqa: F811
    """ can pwdtk handle a non default pwd hash """
    user = two_users[0]
    username = user.username
    password = user.raw_password
    print("user %s with %r" % (user, password))
    hashers = settings.PASSWORD_HASHERS
    print("hashers:\n%s" % "\n".join(str(hasher) for hasher in hashers))
    salt = ""
    if "django.contrib.auth.hashers.UnsaltedMD5PasswordHasher" in hashers:
        hasher_cls_name = ("django.contrib.auth.hashers"
                           ".UnsaltedMD5PasswordHasher")
    else:
        hasher_cls_name = hashers[-1]
        if hasher_cls_name.endswith("CryptPasswordHasher"):
            salt = "ab"

    assert hasher_cls_name != hashers[0]
    hasher_modname, hasher_name = hasher_cls_name.rsplit(".", 1)
    hasher_mod = importlib.import_module(hasher_modname)
    print("try to import %s:%s" % (hasher_modname, hasher_name))
    hasher = getattr(hasher_mod, hasher_name)()
    print("hasher = ", hasher)
    passwd_hash = hasher.encode(password, salt)
    print("passwd: %s -> %r" % (password, passwd_hash))

    user.password = passwd_hash
    user.save()

    user = User.objects.get(username=username)
    assert user.check_password(password)



"""
Django settings for pwdtk.testproject.dj21 project.

Generated by 'django-admin startproject' using Django 2.0.9.

For more information on this file, see
https://docs.djangoproject.com/en/2.0/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/2.0/ref/settings/
"""

import os

from django.conf import global_settings


# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

TOP_DIR = os.environ.get("PWDTK_TOPDIR",
                         os.path.dirname(os.path.dirname(BASE_DIR)))


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/2.0/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = 'pel7g&u8r!&f^3b==-4z3y&fba0&*km@f-*chz%p*sidqy!*i9'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = []


AUTHENTICATION_BACKENDS = [
    'django.contrib.auth.backends.ModelBackend',
    ]

# Application definition

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'pwdtk',
]
if os.environ.get('USE_DJANGO_EXTENSIONS', "no").lower()[:1] in "yt1":
    INSTALLED_APPS += [
        "django_extensions"
        ]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

ROOT_URLCONF = 'pwdtk.testproject.dj21.urls'

# PWDTK specifics
# For MH authentification back end
# TODO: refactor and move into separate app
import pwdtk.settings  # noqa E402
pwdtk.settings.add_backend(AUTHENTICATION_BACKENDS)
pwdtk.settings.add_middlewares(MIDDLEWARE)
from pwdtk.settings import *  # noqa F401

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'pwdtk.testproject.dj21.wsgi.application'


# Database
# https://docs.djangoproject.com/en/2.0/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(
            TOP_DIR,
            os.environ.get('PWDTK_DB_FILE', 'db21.sqlite3')),
    }
}


# Password validation
# https://docs.djangoproject.com/en/2.0/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.'
                'UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.'
                'MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.'
                'CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.'
                'NumericPasswordValidator',
    },
]


# Internationalization
# https://docs.djangoproject.com/en/2.0/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True

PASSWORD_HASHERS = list(global_settings.PASSWORD_HASHERS)

if not any("UnsaltedMD5PasswordHasher" in val for val in PASSWORD_HASHERS):
    PASSWORD_HASHERS.append(
         'django.contrib.auth.hashers.UnsaltedMD5PasswordHasher'
        )


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/2.0/howto/static-files/

STATIC_URL = '/static/'

TEST_RUNNER = 'pwdtk.testproject.pytest_runner.PytestTestRunner'

from pwdtk.testproject.logcfg import LOGGING  # noqa: F401



from __future__ import absolute_import
from __future__ import print_function

from django.contrib import admin

from pwdtk.models import PwdData


@admin.register(PwdData)
class PwdDataAdmin(admin.ModelAdmin):
    list_display = (
        'id',
        'user',
        'locked',
        'failed_logins',
        'fail_time',
        'last_change_time',
        'must_renew',)
    search_fields = ('id', 'user')



from __future__ import absolute_import
from __future__ import print_function

import datetime
import logging

import django
import pytest


from builtins import range
from django.contrib.auth import get_user_model
from django.contrib import auth
from django.test import Client
from django.utils import timezone

from pwdtk.tests.fixtures import two_users  # noqa: F401
from pwdtk.helpers import PwdtkSettings


logger = logging.getLogger(__name__)

AUTH_URL = PwdtkSettings.PWDTK_TEST_ADMIN_URL
User = get_user_model()


def do_login(client, data, use_good_password=True, shall_pass=None):
    """ helper to simulate logins via the admin login form.
        Logins with good or bad password can be simulated.
        Checks will be performed to see whether login success / failure is
        as expected.
        :param client: client object
        :param data: form data (username, password, csrf_token, ...)
        :param use_good_password: if set to false a bad password will be used
        :param shall_pass:  indicates the expacted behaviour
                            (failed login, passed login)
                            If set to None, then it is set to use_good_password
    """
    password = data['password']

    if not use_good_password:
        data['password'] = password + 'a'

    if shall_pass is None:
        shall_pass = use_good_password

    url = AUTH_URL + "login/?next=" + AUTH_URL
    logger.debug("loginurl = %s", url)
    resp = client.post(url, data=data)

    user = auth.get_user(client)
    if django.VERSION < (1, 10):
        is_authenticated = user.is_authenticated()
    else:
        is_authenticated = user.is_authenticated
    if shall_pass:
        assert is_authenticated
    else:
        assert not is_authenticated
        data['password'] = password  # restore password

    return resp


def do_logout(client):
    """ helper to logout a user via the admin web form """
    url = AUTH_URL + "logout/"
    resp = client.post(url)
    status_code = int(resp.status_code)
    assert status_code in [200, 302]
    logger.debug("logout successful")


@pytest.mark.django_db
def test_login_no_form(two_users):  # noqa: F811
    """ client login without the login url """
    browser = "Mozilla/5.0"
    client = Client(browser=browser)

    user = two_users[0]
    username = user.username
    password = user.raw_password
    # First successful login without form
    loggedin = client.login(username=username, password=password)
    print("%s %s -> %s" % (username, password, loggedin))
    assert loggedin is True

    # login with bad password fails without form
    bad_pwd = password + 'a'
    loggedin = client.login(username=username, password=bad_pwd)
    print("%s %s -> %s" % (username, bad_pwd, loggedin))
    assert loggedin is False


@pytest.mark.django_db
def test_login(two_users):  # noqa: F811
    """ login via the login url """
    browser = "Mozilla/5.0"
    client = Client(browser=browser)

    user = two_users[0]
    username = user.username
    password = user.raw_password

    # go to a login page and fetch csrf token
    url = AUTH_URL + "login/?next=" + AUTH_URL
    logger.debug("loginurl = %s", url)
    resp = client.get(url)
    assert hasattr(resp, 'cookies')
    csrf_token = resp.cookies.get('csrf_token')
    logger.debug("token %r", csrf_token)

    # prepare post_data
    data = dict(
        username=username,
        password=password,
        csrfmiddlewaretoken=csrf_token,
        )

    # login once
    do_login(client, data)
    do_logout(client)

    failure_limit = PwdtkSettings.PWDTK_USER_FAILURE_LIMIT
    if not failure_limit:
        logger.debug("PWDTK_USER_FAILURE_LIMIT not set. will skip test")
        return

    # only two bad logins
    for cnt in range(failure_limit - 1):
        logger.debug("bad login %d", cnt)
        resp = do_login(client, data, use_good_password=False)
        assert resp.status_code == 200

    # Now logging in should still work
    do_login(client, data)

    do_logout(client)
    # now two bad logins
    for cnt in range(failure_limit - 1):
        logger.debug("bad login %d", cnt)
        resp = do_login(client, data, use_good_password=False)
        assert resp.status_code == 200

    # Now the final bad login
    resp = do_login(client, data, use_good_password=False)
    assert resp.status_code == 403

    # Now logging in should fail
    do_login(client, data, shall_pass=False)

    # now let's check out the lockout delay
    # pwdtk data will be populate after first login attempt
    user = User.objects.get(username=username)
    pwdtk_data = user.pwdtk_data
    assert pwdtk_data.locked
    logger.debug("AGE %i", pwdtk_data.fail_age)


@pytest.mark.django_db
def test_pwd_expire(two_users):  # noqa: F811
    """ test whether a password renewal is demanded if a password
        has not been changed for a given time.
    """

    browser = "Mozilla/5.0"
    client = Client(browser=browser)

    user = two_users[0]
    username = user.username
    password = user.raw_password

    # # go to a login page and fetch csrf token
    url = AUTH_URL + "login/?next=" + AUTH_URL

    resp = client.get(url)

    csrf_token = resp.cookies.get('csrf_token')
    # prepare post_data

    password += '1'
    user.set_password(password)
    user.save()

    data = dict(
        username=username,
        password=password,
        csrfmiddlewaretoken=csrf_token,
        )
    do_login(client, data)

    # pwdtk data will be populate after first login
    user = User.objects.get(username=username)
    pwdtk_data = user.pwdtk_data

    # make passwd obsolete
    pwdtk_data.last_change_time = (
        timezone.now() -
        datetime.timedelta(seconds=PwdtkSettings.PWDTK_PASSWD_AGE)
    )
    pwdtk_data.save()
    assert pwdtk_data.compute_must_renew()

    # now login should fail
    do_login(client, data)

    user = User.objects.get(username=username)
    assert user.pwdtk_data.must_renew

    user.set_password(password)
    user.save()
    assert user.pwdtk_data.must_renew

    user.set_password(password+"2")
    user.save()
    assert not user.pwdtk_data.must_renew



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pwdtk', '0002_auto_20191105_1610'),
    ]

    operations = [
        migrations.AlterField(
            model_name='pwddata',
            name='username',
            field=models.CharField(max_length=255, unique=True, null=True),
        ),
    ]



import logging

import django
from django import forms
from django import template

from django.contrib.auth import authenticate
import django.contrib.auth as auth
from django.contrib.auth import logout
from django.contrib.auth.decorators import login_required
from django.forms.widgets import PasswordInput
from django.http import HttpResponse
from django.shortcuts import render


logger = logging.getLogger(__name__)

HOME_TEMPLATE = """
welcome to the test space &lt;{{ username }}&gt;<br/>
You are {% if not username %} NOT {% endif %} logged in.<br/>
<br/>
<a href="/protected">Access the protected space</a><br/>
<a href="/admin">admin area</a><br/>
<a href="/login?next=/">Login</a><br/>
<a href="/logout?next=/">Logout</a><br/>
"""

LOGOUT_TEMPLATE = """
You should be logged out ({{ username }}).&gt;<br/>
You are {% if not username %} NOT {% endif %} logged in.<br/>
<br/>
<a href="/">Home</a><br/>
"""

PROTECTED_TEMPLATE = """
Welcome to the protected space.<br/>
You are {{ username }}
This space requires to be authenticated<br/>
<a href="/">Top</a><br/>
<a href="/admin/password_change/">Change password</a><br/>
<a href="/logout?next=/protected">Logout</a><br/>
"""


class LoginForm(forms.Form):
    username = forms.CharField(label='username', max_length=256)
    password = forms.CharField(label='password', max_length=256,
                               widget=PasswordInput)


def myauthenticate(request, **kwargs):
    if django.VERSION < (1, 9):
        return authenticate(**kwargs)
    else:
        return authenticate(request, **kwargs)


def home(request):
    """ a small home page for debugging / testing """
    tmplt = template.Template(HOME_TEMPLATE)
    ctx = template.Context(dict(
        username=request.user.username if request.user else "?",
        ))
    rendered = tmplt.render(ctx)
    return HttpResponse(rendered)


def login_view(request):
    ctx = dict(
        loggedin=False,
        )
    if request.method == 'POST':
        form = LoginForm(request.POST)
        logger.debug("handle login post request")
        if form.is_valid():
            logger.debug("form is valid")
            data = form.cleaned_data
            logger.debug("Data %s", data)
            username = data['username']
            password = data['password']
            user = myauthenticate(request, username=username,
                                  password=password)
            if user is not None:
                logger.debug("pwd OK for %s. will login", user.username)
                auth.login(request, user)
                ctx['loggedin'] = True
                ctx['username'] = user.username
            else:
                logger.debug("login error")
    else:
        logger.debug("should show login form")
        form = LoginForm()
    ctx['form'] = form
    return render(request, 'test/login.html', ctx)


@login_required
def logout_view(request):
    """ a small logout page for debugging / testing """
    user = request.user

    logout(request)
    logger.debug("user %s should be logged out now", user.username)
    tmplt = template.Template(LOGOUT_TEMPLATE)
    ctx = template.Context(dict(
        username=request.user.username if request.user else "?",
        ))
    rendered = tmplt.render(ctx)
    return HttpResponse(rendered)


@login_required
def protected(request):
    """ a small page for debugging / testing, that requires authentication """
    tmplt = template.Template(PROTECTED_TEMPLATE)
    ctx = template.Context(dict(
        username=request.user.username if request.user else "?",
        ))
    rendered = tmplt.render(ctx)
    return HttpResponse(rendered)



from __future__ import absolute_import


# third party modules
from setuptools import find_packages
from setuptools import setup

install_requires = [
    "django-jsonfield",
    "future",
    "minibelt",
    "python-dateutil",
    "pytz",
    ]


long_description="""package to tune django password authentification

It spans a rather wide range of python and django versions and supports:
- temporary lockout after too many bad logins.
- max life time for passwords
""",

setup(
    name="django-pwdtk",
    version="0.3.0",
    description="package to tune django password authentification",
    #long_description=long_description,
    #long_description_content_type="text/x-rst",
    classifiers=[
        "Development Status :: 3 - Alpha",
        ],
    keywords="django authentification password",
    url="https://github.com/mhcomm/django-pwdtk",
    author="MHComm",
    author_email="info@mhcomm.fr",
    license="MIT",
    packages=find_packages(),
    scripts=[],
    entry_points={
        "console_scripts": [
          ]
    },
    project_urls={
      "Documentation": "https://github.com/mhcomm/django-pwdtk",
      "Source": "https://github.com/mhcomm/django-pwdtk",
      "SayThanks": "https://github.com/mhcomm",
      "Funding": "https://donate.pypi.org",
      "Tracker": "https://github.com/mhcomm/django-pwdtk/issues",
    },
    python_requires=">=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*, !=3.4.*, <4",
    setup_requires=["pytest-runner"],
    tests_require=[
        "pytest",
        ],
    include_package_data=True,
    )



from __future__ import absolute_import
from __future__ import print_function

import django
print(django.VERSION)
if django.VERSION < (1, 9):
    from pwdtk.testproject.dj18.settings import *  # noqa: F401, F403
elif django.VERSION < (2, ):
    from pwdtk.testproject.dj1.settings import *  # noqa: F401, F403
else:
    from pwdtk.testproject.dj21.settings import *  # noqa: F401, F403



from __future__ import absolute_import
from __future__ import print_function

import logging

import pytest

from django.contrib.auth import get_user_model


logger = logging.getLogger(__name__)


@pytest.fixture
def two_users():
    """ create two users with two passwords """
    User = get_user_model()
    users = []
    for ctr in range(2):
        username = 'user%d' % ctr
        passwd = 'pwd%d' % ctr
        usr = User(username=username)
        usr.save()
        usr.set_password(passwd)
        usr.is_staff = True
        usr.save()
        usr.raw_password = passwd
        logger.debug("create user %r %r", username, passwd)
        users.append(usr)

    return users



"""dj18 URL Configuration

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/1.8/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  url(r'^$', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  url(r'^$', Home.as_view(), name='home')
Including another URLconf
    1. Add a URL to urlpatterns:  url(r'^blog/', include('blog.urls'))
"""
import django.contrib.auth.views

from django.conf.urls import include
from django.conf.urls import url
from django.contrib import admin

from pwdtk.testviews import home
from pwdtk.testviews import logout_view
from pwdtk.testviews import protected


urlpatterns = [
    url(r'^$', home, name='pwdtk_test_home'),
    url(r'^login', django.contrib.auth.views.login,
        name='pwdtk_test_login'),
    url(r'^logout', logout_view,
        name='pwdtk_test_logout'),
    url(r'^protected', protected,
        name='pwdtk_test_protected'),
    url(r'^admin/', include(admin.site.urls)),
    url(r'^accounts/password_change/$',
        django.contrib.auth.views.password_change,
        name='password_change'),
    url(r'^accounts/password_change/done/$',
        django.contrib.auth.views.password_change_done,
        name='password_change_done'),
]
